#include <windows.h>

#include "config.h"

APP_ICON ICON DISCARDABLE "icon/nitroshare.ico"

// Build a comma-separated version
#define PROJECT_VERSION_RC \
    PROJECT_VERSION_MAJOR, \
    PROJECT_VERSION_MINOR, \
    PROJECT_VERSION_PATCH, \
    0

VS_VERSION_INFO VERSIONINFO
    FILEVERSION    PROJECT_VERSION_RC
    PRODUCTVERSION PROJECT_VERSION_RC
{
    BLOCK "StringFileInfo"
    {
        BLOCK "040904b0"
        {
            VALUE "CompanyName",      PROJECT_AUTHOR "\0"
            VALUE "FileDescription",  PROJECT_DESCRIPTION "\0"
            VALUE "FileVersion",      PROJECT_VERSION "\0"
            VALUE "InternalName",     PROJECT_NAME "\0"
            VALUE "LegalCopyright",   "Copyright (c) 2016 " PROJECT_AUTHOR "\0"
            VALUE "OriginalFilename", PROJECT_NAME ".exe\0"
            VALUE "ProductName",      PROJECT_NAME "\0"
            VALUE "ProductVersion",   PROJECT_VERSION "\0"
        }
    }
    BLOCK "VarFileInfo"
    {
        VALUE "Translation", 0x409, 1252
    }
}
